MODULE bos
  USE kinds, ONLY: wp => dp
  USE constants, ONLY: pi, bohr
  
  IMPLICIT NONE

  TYPE, PUBLIC :: bo
    PRIVATE
    REAL (KIND=wp) :: beta, re
  END TYPE bo

  PUBLIC :: bo_set,  &
            bo_eval, &
            bo_eval_bl, &
            bo_get_grid, &
            bo_get_edggrid, &
            morse!, &
            !bo2bl!, bl2bo

  CONTAINS


  SUBROUTINE bo_set ( mybo, beta, re)
    REAL (KIND=wp), INTENT (IN) :: beta, re
    TYPE (bo), INTENT(OUT) :: mybo

    mybo%beta = beta
    mybo%re = re

  END SUBROUTINE bo_set


  FUNCTION bo_eval ( mybo, r )
    REAL (KIND=wp) :: bo_eval
    TYPE (bo), INTENT(IN) :: mybo
    REAL (KIND=wp), INTENT (IN) :: r

    bo_eval = EXP ( - mybo%beta * (r - mybo%re) )

  END FUNCTION bo_eval


  FUNCTION bo_eval_bl ( mybo, n )
    REAL (KIND=wp) :: bo_eval_bl
    TYPE(bo) , INTENT(IN) :: mybo
    REAL (KIND=wp), INTENT(IN) :: n

    ! if n > EXP(beta*re) error

    bo_eval_bl = mybo%re - LOG( n ) / mybo%beta

  END FUNCTION bo_eval_bl

  
  SUBROUTINE bo_get_grid ( mybo, ar)
    TYPE (bo), INTENT(IN) :: mybo
    REAL (KIND=wp), DIMENSION(:), INTENT (OUT) :: ar
    REAL (KIND=wp) :: dn
    INTEGER :: i, n

    n = SIZE ( ar )
    dn = EXP(mybo%beta*mybo%re)/n
    
    DO i = 1, n
      ar (i) = (i-1) * dn
      !PRINT *, ar(i)
    ENDDO

  END SUBROUTINE bo_get_grid

  SUBROUTINE bo_get_edggrid ( mybo, ar, rmin, rmax)
    TYPE (bo), INTENT(IN) :: mybo
    REAL (KIND=wp), INTENT(IN) :: rmin, rmax
    REAL (KIND=wp), DIMENSION(:), INTENT (OUT) :: ar
    REAL (KIND=wp) :: dn
    INTEGER :: i, n

    n = SIZE ( ar )
    dn = (EXP(-mybo%beta*(rmin - mybo%re)) - EXP(-mybo%beta*(rmax - mybo%re)) )/(n-1)
    
    DO i = 1, n
      ar (i) = EXP(-mybo%beta*(rmax - mybo%re)) + (i-1) * dn
      !PRINT *, ar(i)
    ENDDO

  END SUBROUTINE bo_get_edggrid

  FUNCTION morse ( r, de, re, alpha)
    REAL (KIND=wp), INTENT(IN) :: r, de, re, alpha
    REAL (KIND=wp) :: morse

    morse = de * (1.0_wp - exp(-alpha * (r - re) ) )**2.0_wp

  END FUNCTION morse
!
!  REAL (KIND=wp) :: de, beta, re
!  INTEGER :: i, n
!  REAL (KIND=wp) :: rf, dr
!  REAL (KIND=wp) :: dn
!
!  ! values from marston89_3571
!  re = 1.40201_wp
!  de = 0.1744_wp
!  beta = 1.02764_wp
!
!  PRINT *, "beta:", beta
!  PRINT *, "BO grid edge:", EXP(beta*re)
!
!  ! reset beta to ram-BO variables
!  re = 2.10
!  beta = LOG(1.5_wp)/re
!
!  PRINT *, "beta:", beta
!  PRINT *, "BO grid edge:", EXP(beta*re)
!
!  ! test grid
!  n = 16
!  rf = 12.0_wp
!  dr = rf/n
!
!  dn = EXP(beta*re)/n
!
!  DO i = 1, n
!    PRINT "(I5,2F12.4)", i, (i - 1) * dr, - LOG ( (i-1) * dn )/beta + re
!  ENDDO


END MODULE bos
